@@ -4,15 +4,15 @@ package JSON::Schema::Draft201909::Vocabulary;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Base role for JSON Schema vocabulary classes
 
-our $VERSION = '0.026';
+our $VERSION = '0.027';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
+use strictures 2;
 use JSON::Schema::Draft201909::Utilities qw(jsonp assert_keyword_type);
 use Moo::Role;
-use strictures 2;
 use namespace::clean;
 
 requires qw(vocabulary keywords);
@@ -33,7 +33,7 @@ sub traverse_array_schemas {
   my ($self, $schema, $state) = @_;
 
   return if not assert_keyword_type($state, $schema, 'array');
-  return E($state, '"%s" array is empty') if not @{$schema->{$state->{keyword}}};
+  return E($state, '%s array is empty', $state->{keyword}) if not @{$schema->{$state->{keyword}}};
 
   foreach my $idx (0 .. $#{$schema->{$state->{keyword}}}) {
     $state->{evaluator}->_traverse($schema->{$state->{keyword}}[$idx],
@@ -71,7 +71,7 @@ JSON::Schema::Draft201909::Vocabulary - Base role for JSON Schema vocabulary cla
 
 =head1 VERSION
 
-version 0.026
+version 0.027
 
 =head1 SYNOPSIS
 
